from html.parser import HTMLParser

class Dex_K8S_OAuth2_Parser(object):
    """
    Class object for parsing out useful informaton from HTTP/HTML responses
    from Dex. Yes this is ugly, but I want to programmatically auth users
    to multiple Kubernetes cluster. So far this is the only way I have found.
    """

    @staticmethod
    def get_ldap_authentication_uri(html_content, settings):
        """
        Extract the URI for authenticating against the LDAP connector as well
        as the auth_code for the token request.
        """

        links = []
        class Parse_Authentication_Response(HTMLParser):
            def handle_starttag(self, tag, attrs):
                if tag != 'a':
                    return
                attr = dict(attrs)
                links.append(attr)

        parser = Parse_Authentication_Response()
        parser.feed(html_content)

        ldap_uri = None
        for link in links:
            if link['href'].startswith('/auth/ldap'):
                ldap_uri = link['href']
                break

        return 'https://{0}:{1}{2}'.format(
            settings.dex.host,
            settings.dex.https_port,
            ldap_uri
        )



import re
from base64 import b64encode
from requests_oauthlib import OAuth2Session
from dex_k8s_client.oauth2.parser import Dex_K8S_OAuth2_Parser
from dex_k8s_client.k8s.kubeconfig import Dex_K8S_KubeConfig
from dex_k8s_client.oauth2.token import Dex_K8S_OAuth2_Token

class Dex_K8S_OAuth2_Client(object):
    """
    Client for making requests to Dex OAuth2 authorization URLs.
    """
    def __init__(self, settings):
        self.settings = settings

        # HTML parser
        self.parser = Dex_K8S_OAuth2_Parser

        # SSL verification
        self.verify_ssl = getattr(self.settings.dex, 'ca_cert', True)

    def _session(self, client_id):
        """
        Initiate an OAuth2 session, get an auth_url and state.
        """
        session = OAuth2Session(client_id,
            redirect_uri = self.settings.dex.oauth2.redirect_uri,
            scope        = self.settings.dex.oauth2.scope
        )

        # Get an authorization URL and state
        auth_url, state = session.authorization_url(self.settings.dex.auth_url,
            access_type=self.settings.dex.oauth2.access_type,
            prompt=self.settings.dex.oauth2.prompt)
        return auth_url, state, session

    def _authorize(self, client_id):
        """
        Make an authorization request.
        """
        return self._session(client_id)

    def _get_ldap_authentication_uri(self, auth_url, session):
        """
        Get the URL for making POST authentication requests.
        """
        response = session.get(auth_url, verify=self.verify_ssl)

        if not response.status_code == 200:
            raise Exception('Failed to get authentication URL: auth_url="{}", http_error="{}"'.format(
                auth_url, response.text
            ))

        # Get the URL for making POST requests for LDAP auth
        return self.parser.get_ldap_authentication_uri(
            response.text, self.settings)

    def _login(self, ldap_uri, user_email, user_password, session):
        """
        Login a user.
        """

        response = session.post(ldap_uri, data={
            'login': user_email,
            'password': user_password
        }, verify=self.verify_ssl, allow_redirects=False)

        if not response.status_code == 303:
            raise Exception('Failed to login: {}'.format(response.text))

        # Get the auth code from the approval endpoint
        response = session.get('{}{}'.format(self.settings.dex.base_url,
            response.headers['Location']),
            allow_redirects=False,
            verify=self.verify_ssl)

        return re.search(r"[/]callback[?]code=(\w+)", response.text).group(1)

    def get_token(self, client_id, client_secret, user_email, user_password):
        """
        Get an OAuth2 token for a particular client/user.
        """
        auth_url, state, session = self._authorize(client_id)

        # Get the URL for making POST requests for the LDAP connector
        ldap_uri = self._get_ldap_authentication_uri(auth_url, session)

        # Log the user in and get an auth_code
        auth_code = self._login(
            ldap_uri, user_email, user_password, session)

        # Get and return the token
        token = session.fetch_token(self.settings.dex.token_url,
            code=auth_code,
            client_secret=client_secret, verify=self.verify_ssl)

        return Dex_K8S_OAuth2_Token(token, client_id, session, self.settings)

    def get_kubeconfig(self, client_id, client_secret, user_email, user_password):
        """
        Retrieve a user's kubectl config file.
        """
        token = self.get_token(client_id, client_secret, user_email, user_password).json()

        cluster_ca = self.settings.cluster.read_ca_cert
        cluster_ca = None
        with open(self.settings.cluster.ca_cert, 'rb') as f:
            cluster_ca = f.read()

        return Dex_K8S_KubeConfig.from_template(**{
            'cluster_name': self.settings.cluster.name,
            'cluster_ca': b64encode(cluster_ca).decode(),
            'client_id': client_id,
            'client_secret': client_secret,
            'email': user_email,
            'api_url': self.settings.cluster.api_url,
            'issuer_url': self.settings.dex.issuer_url,
            'refresh_token': token['refresh_token'],
            'id_token': token['id_token']
        })



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from dex_k8s_client.dexidp.dex.api import v2_14_0_pb2 as dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2


class DexStub(object):
  """Dex represents the dex gRPC service.
  """

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.CreateClient = channel.unary_unary(
        '/api.Dex/CreateClient',
        request_serializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.CreateClientReq.SerializeToString,
        response_deserializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.CreateClientResp.FromString,
        )
    self.UpdateClient = channel.unary_unary(
        '/api.Dex/UpdateClient',
        request_serializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.UpdateClientReq.SerializeToString,
        response_deserializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.UpdateClientResp.FromString,
        )
    self.DeleteClient = channel.unary_unary(
        '/api.Dex/DeleteClient',
        request_serializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.DeleteClientReq.SerializeToString,
        response_deserializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.DeleteClientResp.FromString,
        )
    self.CreatePassword = channel.unary_unary(
        '/api.Dex/CreatePassword',
        request_serializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.CreatePasswordReq.SerializeToString,
        response_deserializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.CreatePasswordResp.FromString,
        )
    self.UpdatePassword = channel.unary_unary(
        '/api.Dex/UpdatePassword',
        request_serializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.UpdatePasswordReq.SerializeToString,
        response_deserializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.UpdatePasswordResp.FromString,
        )
    self.DeletePassword = channel.unary_unary(
        '/api.Dex/DeletePassword',
        request_serializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.DeletePasswordReq.SerializeToString,
        response_deserializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.DeletePasswordResp.FromString,
        )
    self.ListPasswords = channel.unary_unary(
        '/api.Dex/ListPasswords',
        request_serializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.ListPasswordReq.SerializeToString,
        response_deserializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.ListPasswordResp.FromString,
        )
    self.GetVersion = channel.unary_unary(
        '/api.Dex/GetVersion',
        request_serializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.VersionReq.SerializeToString,
        response_deserializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.VersionResp.FromString,
        )
    self.ListRefresh = channel.unary_unary(
        '/api.Dex/ListRefresh',
        request_serializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.ListRefreshReq.SerializeToString,
        response_deserializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.ListRefreshResp.FromString,
        )
    self.RevokeRefresh = channel.unary_unary(
        '/api.Dex/RevokeRefresh',
        request_serializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.RevokeRefreshReq.SerializeToString,
        response_deserializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.RevokeRefreshResp.FromString,
        )


class DexServicer(object):
  """Dex represents the dex gRPC service.
  """

  def CreateClient(self, request, context):
    """CreateClient creates a client.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def UpdateClient(self, request, context):
    """UpdateClient updates an existing client
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def DeleteClient(self, request, context):
    """DeleteClient deletes the provided client.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def CreatePassword(self, request, context):
    """CreatePassword creates a password.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def UpdatePassword(self, request, context):
    """UpdatePassword modifies existing password.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def DeletePassword(self, request, context):
    """DeletePassword deletes the password.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ListPasswords(self, request, context):
    """ListPassword lists all password entries.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetVersion(self, request, context):
    """GetVersion returns version information of the server.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ListRefresh(self, request, context):
    """ListRefresh lists all the refresh token entries for a particular user.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def RevokeRefresh(self, request, context):
    """RevokeRefresh revokes the refresh token for the provided user-client pair.

    Note that each user-client pair can have only one refresh token at a time.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_DexServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'CreateClient': grpc.unary_unary_rpc_method_handler(
          servicer.CreateClient,
          request_deserializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.CreateClientReq.FromString,
          response_serializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.CreateClientResp.SerializeToString,
      ),
      'UpdateClient': grpc.unary_unary_rpc_method_handler(
          servicer.UpdateClient,
          request_deserializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.UpdateClientReq.FromString,
          response_serializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.UpdateClientResp.SerializeToString,
      ),
      'DeleteClient': grpc.unary_unary_rpc_method_handler(
          servicer.DeleteClient,
          request_deserializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.DeleteClientReq.FromString,
          response_serializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.DeleteClientResp.SerializeToString,
      ),
      'CreatePassword': grpc.unary_unary_rpc_method_handler(
          servicer.CreatePassword,
          request_deserializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.CreatePasswordReq.FromString,
          response_serializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.CreatePasswordResp.SerializeToString,
      ),
      'UpdatePassword': grpc.unary_unary_rpc_method_handler(
          servicer.UpdatePassword,
          request_deserializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.UpdatePasswordReq.FromString,
          response_serializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.UpdatePasswordResp.SerializeToString,
      ),
      'DeletePassword': grpc.unary_unary_rpc_method_handler(
          servicer.DeletePassword,
          request_deserializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.DeletePasswordReq.FromString,
          response_serializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.DeletePasswordResp.SerializeToString,
      ),
      'ListPasswords': grpc.unary_unary_rpc_method_handler(
          servicer.ListPasswords,
          request_deserializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.ListPasswordReq.FromString,
          response_serializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.ListPasswordResp.SerializeToString,
      ),
      'GetVersion': grpc.unary_unary_rpc_method_handler(
          servicer.GetVersion,
          request_deserializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.VersionReq.FromString,
          response_serializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.VersionResp.SerializeToString,
      ),
      'ListRefresh': grpc.unary_unary_rpc_method_handler(
          servicer.ListRefresh,
          request_deserializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.ListRefreshReq.FromString,
          response_serializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.ListRefreshResp.SerializeToString,
      ),
      'RevokeRefresh': grpc.unary_unary_rpc_method_handler(
          servicer.RevokeRefresh,
          request_deserializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.RevokeRefreshReq.FromString,
          response_serializer=dex__k8s__client_dot_dexidp_dot_dex_dot_api_dot_v2__14__0__pb2.RevokeRefreshResp.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'api.Dex', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



import unittest
from base64 import b64encode
from dex_k8s_client.k8s.kubeconfig import Dex_K8S_KubeConfig

class Dex_K8S_KubeConfig_Test(unittest.TestCase):
    """Tests for `k8s/kubeconfig.py`."""

    def test_create_kubeconfig_from_template(self):
        """ Test creating a new kubeconfig from template string """
        kubeconfig = Dex_K8S_KubeConfig.from_template(**{
            'cluster_name': 'cluster_name',
            'cluster_ca': b64encode(b'cluster_ca').decode(),
            'client_id': 'client_id',
            'client_secret': 'client_secret',
            'email': 'user_email',
            'api_url': 'api_url',
            'issuer_url': 'issuer_url',
            'refresh_token': 'refresh_token',
            'id_token': 'id_token'
        })

        self.assertIsInstance(kubeconfig, dict)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: dex_k8s_client/dexidp/dex/api/v2_14_0.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='dex_k8s_client/dexidp/dex/api/v2_14_0.proto',
  package='api',
  syntax='proto3',
  serialized_options=_b('\n\022com.coreos.dex.api'),
  serialized_pb=_b('\n+dex_k8s_client/dexidp/dex/api/v2_14_0.proto\x12\x03\x61pi\"\x82\x01\n\x06\x43lient\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0e\n\x06secret\x18\x02 \x01(\t\x12\x15\n\rredirect_uris\x18\x03 \x03(\t\x12\x15\n\rtrusted_peers\x18\x04 \x03(\t\x12\x0e\n\x06public\x18\x05 \x01(\x08\x12\x0c\n\x04name\x18\x06 \x01(\t\x12\x10\n\x08logo_url\x18\x07 \x01(\t\".\n\x0f\x43reateClientReq\x12\x1b\n\x06\x63lient\x18\x01 \x01(\x0b\x32\x0b.api.Client\"G\n\x10\x43reateClientResp\x12\x16\n\x0e\x61lready_exists\x18\x01 \x01(\x08\x12\x1b\n\x06\x63lient\x18\x02 \x01(\x0b\x32\x0b.api.Client\"\x1d\n\x0f\x44\x65leteClientReq\x12\n\n\x02id\x18\x01 \x01(\t\"%\n\x10\x44\x65leteClientResp\x12\x11\n\tnot_found\x18\x01 \x01(\x08\"k\n\x0fUpdateClientReq\x12\n\n\x02id\x18\x01 \x01(\t\x12\x15\n\rredirect_uris\x18\x02 \x03(\t\x12\x15\n\rtrusted_peers\x18\x03 \x03(\t\x12\x0c\n\x04name\x18\x04 \x01(\t\x12\x10\n\x08logo_url\x18\x05 \x01(\t\"%\n\x10UpdateClientResp\x12\x11\n\tnot_found\x18\x01 \x01(\x08\"J\n\x08Password\x12\r\n\x05\x65mail\x18\x01 \x01(\t\x12\x0c\n\x04hash\x18\x02 \x01(\x0c\x12\x10\n\x08username\x18\x03 \x01(\t\x12\x0f\n\x07user_id\x18\x04 \x01(\t\"4\n\x11\x43reatePasswordReq\x12\x1f\n\x08password\x18\x01 \x01(\x0b\x32\r.api.Password\",\n\x12\x43reatePasswordResp\x12\x16\n\x0e\x61lready_exists\x18\x01 \x01(\x08\"J\n\x11UpdatePasswordReq\x12\r\n\x05\x65mail\x18\x01 \x01(\t\x12\x10\n\x08new_hash\x18\x02 \x01(\x0c\x12\x14\n\x0cnew_username\x18\x03 \x01(\t\"\'\n\x12UpdatePasswordResp\x12\x11\n\tnot_found\x18\x01 \x01(\x08\"\"\n\x11\x44\x65letePasswordReq\x12\r\n\x05\x65mail\x18\x01 \x01(\t\"\'\n\x12\x44\x65letePasswordResp\x12\x11\n\tnot_found\x18\x01 \x01(\x08\"\x11\n\x0fListPasswordReq\"4\n\x10ListPasswordResp\x12 \n\tpasswords\x18\x01 \x03(\x0b\x32\r.api.Password\"\x0c\n\nVersionReq\"*\n\x0bVersionResp\x12\x0e\n\x06server\x18\x01 \x01(\t\x12\x0b\n\x03\x61pi\x18\x02 \x01(\x05\"W\n\x0fRefreshTokenRef\x12\n\n\x02id\x18\x01 \x01(\t\x12\x11\n\tclient_id\x18\x02 \x01(\t\x12\x12\n\ncreated_at\x18\x05 \x01(\x03\x12\x11\n\tlast_used\x18\x06 \x01(\x03\"!\n\x0eListRefreshReq\x12\x0f\n\x07user_id\x18\x01 \x01(\t\"?\n\x0fListRefreshResp\x12,\n\x0erefresh_tokens\x18\x01 \x03(\x0b\x32\x14.api.RefreshTokenRef\"6\n\x10RevokeRefreshReq\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x11\n\tclient_id\x18\x02 \x01(\t\"&\n\x11RevokeRefreshResp\x12\x11\n\tnot_found\x18\x01 \x01(\x08\x32\x82\x05\n\x03\x44\x65x\x12=\n\x0c\x43reateClient\x12\x14.api.CreateClientReq\x1a\x15.api.CreateClientResp\"\x00\x12=\n\x0cUpdateClient\x12\x14.api.UpdateClientReq\x1a\x15.api.UpdateClientResp\"\x00\x12=\n\x0c\x44\x65leteClient\x12\x14.api.DeleteClientReq\x1a\x15.api.DeleteClientResp\"\x00\x12\x43\n\x0e\x43reatePassword\x12\x16.api.CreatePasswordReq\x1a\x17.api.CreatePasswordResp\"\x00\x12\x43\n\x0eUpdatePassword\x12\x16.api.UpdatePasswordReq\x1a\x17.api.UpdatePasswordResp\"\x00\x12\x43\n\x0e\x44\x65letePassword\x12\x16.api.DeletePasswordReq\x1a\x17.api.DeletePasswordResp\"\x00\x12>\n\rListPasswords\x12\x14.api.ListPasswordReq\x1a\x15.api.ListPasswordResp\"\x00\x12\x31\n\nGetVersion\x12\x0f.api.VersionReq\x1a\x10.api.VersionResp\"\x00\x12:\n\x0bListRefresh\x12\x13.api.ListRefreshReq\x1a\x14.api.ListRefreshResp\"\x00\x12@\n\rRevokeRefresh\x12\x15.api.RevokeRefreshReq\x1a\x16.api.RevokeRefreshResp\"\x00\x42\x14\n\x12\x63om.coreos.dex.apib\x06proto3')
)




_CLIENT = _descriptor.Descriptor(
  name='Client',
  full_name='api.Client',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='api.Client.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='secret', full_name='api.Client.secret', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='redirect_uris', full_name='api.Client.redirect_uris', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='trusted_peers', full_name='api.Client.trusted_peers', index=3,
      number=4, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='public', full_name='api.Client.public', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name', full_name='api.Client.name', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='logo_url', full_name='api.Client.logo_url', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=53,
  serialized_end=183,
)


_CREATECLIENTREQ = _descriptor.Descriptor(
  name='CreateClientReq',
  full_name='api.CreateClientReq',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='client', full_name='api.CreateClientReq.client', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=185,
  serialized_end=231,
)


_CREATECLIENTRESP = _descriptor.Descriptor(
  name='CreateClientResp',
  full_name='api.CreateClientResp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='already_exists', full_name='api.CreateClientResp.already_exists', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='client', full_name='api.CreateClientResp.client', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=233,
  serialized_end=304,
)


_DELETECLIENTREQ = _descriptor.Descriptor(
  name='DeleteClientReq',
  full_name='api.DeleteClientReq',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='api.DeleteClientReq.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=306,
  serialized_end=335,
)


_DELETECLIENTRESP = _descriptor.Descriptor(
  name='DeleteClientResp',
  full_name='api.DeleteClientResp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='not_found', full_name='api.DeleteClientResp.not_found', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=337,
  serialized_end=374,
)


_UPDATECLIENTREQ = _descriptor.Descriptor(
  name='UpdateClientReq',
  full_name='api.UpdateClientReq',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='api.UpdateClientReq.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='redirect_uris', full_name='api.UpdateClientReq.redirect_uris', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='trusted_peers', full_name='api.UpdateClientReq.trusted_peers', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name', full_name='api.UpdateClientReq.name', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='logo_url', full_name='api.UpdateClientReq.logo_url', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=376,
  serialized_end=483,
)


_UPDATECLIENTRESP = _descriptor.Descriptor(
  name='UpdateClientResp',
  full_name='api.UpdateClientResp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='not_found', full_name='api.UpdateClientResp.not_found', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=485,
  serialized_end=522,
)


_PASSWORD = _descriptor.Descriptor(
  name='Password',
  full_name='api.Password',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='email', full_name='api.Password.email', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hash', full_name='api.Password.hash', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='username', full_name='api.Password.username', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='user_id', full_name='api.Password.user_id', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=524,
  serialized_end=598,
)


_CREATEPASSWORDREQ = _descriptor.Descriptor(
  name='CreatePasswordReq',
  full_name='api.CreatePasswordReq',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='password', full_name='api.CreatePasswordReq.password', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=600,
  serialized_end=652,
)


_CREATEPASSWORDRESP = _descriptor.Descriptor(
  name='CreatePasswordResp',
  full_name='api.CreatePasswordResp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='already_exists', full_name='api.CreatePasswordResp.already_exists', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=654,
  serialized_end=698,
)


_UPDATEPASSWORDREQ = _descriptor.Descriptor(
  name='UpdatePasswordReq',
  full_name='api.UpdatePasswordReq',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='email', full_name='api.UpdatePasswordReq.email', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='new_hash', full_name='api.UpdatePasswordReq.new_hash', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='new_username', full_name='api.UpdatePasswordReq.new_username', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=700,
  serialized_end=774,
)


_UPDATEPASSWORDRESP = _descriptor.Descriptor(
  name='UpdatePasswordResp',
  full_name='api.UpdatePasswordResp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='not_found', full_name='api.UpdatePasswordResp.not_found', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=776,
  serialized_end=815,
)


_DELETEPASSWORDREQ = _descriptor.Descriptor(
  name='DeletePasswordReq',
  full_name='api.DeletePasswordReq',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='email', full_name='api.DeletePasswordReq.email', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=817,
  serialized_end=851,
)


_DELETEPASSWORDRESP = _descriptor.Descriptor(
  name='DeletePasswordResp',
  full_name='api.DeletePasswordResp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='not_found', full_name='api.DeletePasswordResp.not_found', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=853,
  serialized_end=892,
)


_LISTPASSWORDREQ = _descriptor.Descriptor(
  name='ListPasswordReq',
  full_name='api.ListPasswordReq',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=894,
  serialized_end=911,
)


_LISTPASSWORDRESP = _descriptor.Descriptor(
  name='ListPasswordResp',
  full_name='api.ListPasswordResp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='passwords', full_name='api.ListPasswordResp.passwords', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=913,
  serialized_end=965,
)


_VERSIONREQ = _descriptor.Descriptor(
  name='VersionReq',
  full_name='api.VersionReq',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=967,
  serialized_end=979,
)


_VERSIONRESP = _descriptor.Descriptor(
  name='VersionResp',
  full_name='api.VersionResp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='server', full_name='api.VersionResp.server', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='api', full_name='api.VersionResp.api', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=981,
  serialized_end=1023,
)


_REFRESHTOKENREF = _descriptor.Descriptor(
  name='RefreshTokenRef',
  full_name='api.RefreshTokenRef',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='api.RefreshTokenRef.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='client_id', full_name='api.RefreshTokenRef.client_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='created_at', full_name='api.RefreshTokenRef.created_at', index=2,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='last_used', full_name='api.RefreshTokenRef.last_used', index=3,
      number=6, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1025,
  serialized_end=1112,
)


_LISTREFRESHREQ = _descriptor.Descriptor(
  name='ListRefreshReq',
  full_name='api.ListRefreshReq',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='user_id', full_name='api.ListRefreshReq.user_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1114,
  serialized_end=1147,
)


_LISTREFRESHRESP = _descriptor.Descriptor(
  name='ListRefreshResp',
  full_name='api.ListRefreshResp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='refresh_tokens', full_name='api.ListRefreshResp.refresh_tokens', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1149,
  serialized_end=1212,
)


_REVOKEREFRESHREQ = _descriptor.Descriptor(
  name='RevokeRefreshReq',
  full_name='api.RevokeRefreshReq',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='user_id', full_name='api.RevokeRefreshReq.user_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='client_id', full_name='api.RevokeRefreshReq.client_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1214,
  serialized_end=1268,
)


_REVOKEREFRESHRESP = _descriptor.Descriptor(
  name='RevokeRefreshResp',
  full_name='api.RevokeRefreshResp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='not_found', full_name='api.RevokeRefreshResp.not_found', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1270,
  serialized_end=1308,
)

_CREATECLIENTREQ.fields_by_name['client'].message_type = _CLIENT
_CREATECLIENTRESP.fields_by_name['client'].message_type = _CLIENT
_CREATEPASSWORDREQ.fields_by_name['password'].message_type = _PASSWORD
_LISTPASSWORDRESP.fields_by_name['passwords'].message_type = _PASSWORD
_LISTREFRESHRESP.fields_by_name['refresh_tokens'].message_type = _REFRESHTOKENREF
DESCRIPTOR.message_types_by_name['Client'] = _CLIENT
DESCRIPTOR.message_types_by_name['CreateClientReq'] = _CREATECLIENTREQ
DESCRIPTOR.message_types_by_name['CreateClientResp'] = _CREATECLIENTRESP
DESCRIPTOR.message_types_by_name['DeleteClientReq'] = _DELETECLIENTREQ
DESCRIPTOR.message_types_by_name['DeleteClientResp'] = _DELETECLIENTRESP
DESCRIPTOR.message_types_by_name['UpdateClientReq'] = _UPDATECLIENTREQ
DESCRIPTOR.message_types_by_name['UpdateClientResp'] = _UPDATECLIENTRESP
DESCRIPTOR.message_types_by_name['Password'] = _PASSWORD
DESCRIPTOR.message_types_by_name['CreatePasswordReq'] = _CREATEPASSWORDREQ
DESCRIPTOR.message_types_by_name['CreatePasswordResp'] = _CREATEPASSWORDRESP
DESCRIPTOR.message_types_by_name['UpdatePasswordReq'] = _UPDATEPASSWORDREQ
DESCRIPTOR.message_types_by_name['UpdatePasswordResp'] = _UPDATEPASSWORDRESP
DESCRIPTOR.message_types_by_name['DeletePasswordReq'] = _DELETEPASSWORDREQ
DESCRIPTOR.message_types_by_name['DeletePasswordResp'] = _DELETEPASSWORDRESP
DESCRIPTOR.message_types_by_name['ListPasswordReq'] = _LISTPASSWORDREQ
DESCRIPTOR.message_types_by_name['ListPasswordResp'] = _LISTPASSWORDRESP
DESCRIPTOR.message_types_by_name['VersionReq'] = _VERSIONREQ
DESCRIPTOR.message_types_by_name['VersionResp'] = _VERSIONRESP
DESCRIPTOR.message_types_by_name['RefreshTokenRef'] = _REFRESHTOKENREF
DESCRIPTOR.message_types_by_name['ListRefreshReq'] = _LISTREFRESHREQ
DESCRIPTOR.message_types_by_name['ListRefreshResp'] = _LISTREFRESHRESP
DESCRIPTOR.message_types_by_name['RevokeRefreshReq'] = _REVOKEREFRESHREQ
DESCRIPTOR.message_types_by_name['RevokeRefreshResp'] = _REVOKEREFRESHRESP
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Client = _reflection.GeneratedProtocolMessageType('Client', (_message.Message,), dict(
  DESCRIPTOR = _CLIENT,
  __module__ = 'dex_k8s_client.dexidp.dex.api.v2_14_0_pb2'
  # @@protoc_insertion_point(class_scope:api.Client)
  ))
_sym_db.RegisterMessage(Client)

CreateClientReq = _reflection.GeneratedProtocolMessageType('CreateClientReq', (_message.Message,), dict(
  DESCRIPTOR = _CREATECLIENTREQ,
  __module__ = 'dex_k8s_client.dexidp.dex.api.v2_14_0_pb2'
  # @@protoc_insertion_point(class_scope:api.CreateClientReq)
  ))
_sym_db.RegisterMessage(CreateClientReq)

CreateClientResp = _reflection.GeneratedProtocolMessageType('CreateClientResp', (_message.Message,), dict(
  DESCRIPTOR = _CREATECLIENTRESP,
  __module__ = 'dex_k8s_client.dexidp.dex.api.v2_14_0_pb2'
  # @@protoc_insertion_point(class_scope:api.CreateClientResp)
  ))
_sym_db.RegisterMessage(CreateClientResp)

DeleteClientReq = _reflection.GeneratedProtocolMessageType('DeleteClientReq', (_message.Message,), dict(
  DESCRIPTOR = _DELETECLIENTREQ,
  __module__ = 'dex_k8s_client.dexidp.dex.api.v2_14_0_pb2'
  # @@protoc_insertion_point(class_scope:api.DeleteClientReq)
  ))
_sym_db.RegisterMessage(DeleteClientReq)

DeleteClientResp = _reflection.GeneratedProtocolMessageType('DeleteClientResp', (_message.Message,), dict(
  DESCRIPTOR = _DELETECLIENTRESP,
  __module__ = 'dex_k8s_client.dexidp.dex.api.v2_14_0_pb2'
  # @@protoc_insertion_point(class_scope:api.DeleteClientResp)
  ))
_sym_db.RegisterMessage(DeleteClientResp)

UpdateClientReq = _reflection.GeneratedProtocolMessageType('UpdateClientReq', (_message.Message,), dict(
  DESCRIPTOR = _UPDATECLIENTREQ,
  __module__ = 'dex_k8s_client.dexidp.dex.api.v2_14_0_pb2'
  # @@protoc_insertion_point(class_scope:api.UpdateClientReq)
  ))
_sym_db.RegisterMessage(UpdateClientReq)

UpdateClientResp = _reflection.GeneratedProtocolMessageType('UpdateClientResp', (_message.Message,), dict(
  DESCRIPTOR = _UPDATECLIENTRESP,
  __module__ = 'dex_k8s_client.dexidp.dex.api.v2_14_0_pb2'
  # @@protoc_insertion_point(class_scope:api.UpdateClientResp)
  ))
_sym_db.RegisterMessage(UpdateClientResp)

Password = _reflection.GeneratedProtocolMessageType('Password', (_message.Message,), dict(
  DESCRIPTOR = _PASSWORD,
  __module__ = 'dex_k8s_client.dexidp.dex.api.v2_14_0_pb2'
  # @@protoc_insertion_point(class_scope:api.Password)
  ))
_sym_db.RegisterMessage(Password)

CreatePasswordReq = _reflection.GeneratedProtocolMessageType('CreatePasswordReq', (_message.Message,), dict(
  DESCRIPTOR = _CREATEPASSWORDREQ,
  __module__ = 'dex_k8s_client.dexidp.dex.api.v2_14_0_pb2'
  # @@protoc_insertion_point(class_scope:api.CreatePasswordReq)
  ))
_sym_db.RegisterMessage(CreatePasswordReq)

CreatePasswordResp = _reflection.GeneratedProtocolMessageType('CreatePasswordResp', (_message.Message,), dict(
  DESCRIPTOR = _CREATEPASSWORDRESP,
  __module__ = 'dex_k8s_client.dexidp.dex.api.v2_14_0_pb2'
  # @@protoc_insertion_point(class_scope:api.CreatePasswordResp)
  ))
_sym_db.RegisterMessage(CreatePasswordResp)

UpdatePasswordReq = _reflection.GeneratedProtocolMessageType('UpdatePasswordReq', (_message.Message,), dict(
  DESCRIPTOR = _UPDATEPASSWORDREQ,
  __module__ = 'dex_k8s_client.dexidp.dex.api.v2_14_0_pb2'
  # @@protoc_insertion_point(class_scope:api.UpdatePasswordReq)
  ))
_sym_db.RegisterMessage(UpdatePasswordReq)

UpdatePasswordResp = _reflection.GeneratedProtocolMessageType('UpdatePasswordResp', (_message.Message,), dict(
  DESCRIPTOR = _UPDATEPASSWORDRESP,
  __module__ = 'dex_k8s_client.dexidp.dex.api.v2_14_0_pb2'
  # @@protoc_insertion_point(class_scope:api.UpdatePasswordResp)
  ))
_sym_db.RegisterMessage(UpdatePasswordResp)

DeletePasswordReq = _reflection.GeneratedProtocolMessageType('DeletePasswordReq', (_message.Message,), dict(
  DESCRIPTOR = _DELETEPASSWORDREQ,
  __module__ = 'dex_k8s_client.dexidp.dex.api.v2_14_0_pb2'
  # @@protoc_insertion_point(class_scope:api.DeletePasswordReq)
  ))
_sym_db.RegisterMessage(DeletePasswordReq)

DeletePasswordResp = _reflection.GeneratedProtocolMessageType('DeletePasswordResp', (_message.Message,), dict(
  DESCRIPTOR = _DELETEPASSWORDRESP,
  __module__ = 'dex_k8s_client.dexidp.dex.api.v2_14_0_pb2'
  # @@protoc_insertion_point(class_scope:api.DeletePasswordResp)
  ))
_sym_db.RegisterMessage(DeletePasswordResp)

ListPasswordReq = _reflection.GeneratedProtocolMessageType('ListPasswordReq', (_message.Message,), dict(
  DESCRIPTOR = _LISTPASSWORDREQ,
  __module__ = 'dex_k8s_client.dexidp.dex.api.v2_14_0_pb2'
  # @@protoc_insertion_point(class_scope:api.ListPasswordReq)
  ))
_sym_db.RegisterMessage(ListPasswordReq)

ListPasswordResp = _reflection.GeneratedProtocolMessageType('ListPasswordResp', (_message.Message,), dict(
  DESCRIPTOR = _LISTPASSWORDRESP,
  __module__ = 'dex_k8s_client.dexidp.dex.api.v2_14_0_pb2'
  # @@protoc_insertion_point(class_scope:api.ListPasswordResp)
  ))
_sym_db.RegisterMessage(ListPasswordResp)

VersionReq = _reflection.GeneratedProtocolMessageType('VersionReq', (_message.Message,), dict(
  DESCRIPTOR = _VERSIONREQ,
  __module__ = 'dex_k8s_client.dexidp.dex.api.v2_14_0_pb2'
  # @@protoc_insertion_point(class_scope:api.VersionReq)
  ))
_sym_db.RegisterMessage(VersionReq)

VersionResp = _reflection.GeneratedProtocolMessageType('VersionResp', (_message.Message,), dict(
  DESCRIPTOR = _VERSIONRESP,
  __module__ = 'dex_k8s_client.dexidp.dex.api.v2_14_0_pb2'
  # @@protoc_insertion_point(class_scope:api.VersionResp)
  ))
_sym_db.RegisterMessage(VersionResp)

RefreshTokenRef = _reflection.GeneratedProtocolMessageType('RefreshTokenRef', (_message.Message,), dict(
  DESCRIPTOR = _REFRESHTOKENREF,
  __module__ = 'dex_k8s_client.dexidp.dex.api.v2_14_0_pb2'
  # @@protoc_insertion_point(class_scope:api.RefreshTokenRef)
  ))
_sym_db.RegisterMessage(RefreshTokenRef)

ListRefreshReq = _reflection.GeneratedProtocolMessageType('ListRefreshReq', (_message.Message,), dict(
  DESCRIPTOR = _LISTREFRESHREQ,
  __module__ = 'dex_k8s_client.dexidp.dex.api.v2_14_0_pb2'
  # @@protoc_insertion_point(class_scope:api.ListRefreshReq)
  ))
_sym_db.RegisterMessage(ListRefreshReq)

ListRefreshResp = _reflection.GeneratedProtocolMessageType('ListRefreshResp', (_message.Message,), dict(
  DESCRIPTOR = _LISTREFRESHRESP,
  __module__ = 'dex_k8s_client.dexidp.dex.api.v2_14_0_pb2'
  # @@protoc_insertion_point(class_scope:api.ListRefreshResp)
  ))
_sym_db.RegisterMessage(ListRefreshResp)

RevokeRefreshReq = _reflection.GeneratedProtocolMessageType('RevokeRefreshReq', (_message.Message,), dict(
  DESCRIPTOR = _REVOKEREFRESHREQ,
  __module__ = 'dex_k8s_client.dexidp.dex.api.v2_14_0_pb2'
  # @@protoc_insertion_point(class_scope:api.RevokeRefreshReq)
  ))
_sym_db.RegisterMessage(RevokeRefreshReq)

RevokeRefreshResp = _reflection.GeneratedProtocolMessageType('RevokeRefreshResp', (_message.Message,), dict(
  DESCRIPTOR = _REVOKEREFRESHRESP,
  __module__ = 'dex_k8s_client.dexidp.dex.api.v2_14_0_pb2'
  # @@protoc_insertion_point(class_scope:api.RevokeRefreshResp)
  ))
_sym_db.RegisterMessage(RevokeRefreshResp)


DESCRIPTOR._options = None

_DEX = _descriptor.ServiceDescriptor(
  name='Dex',
  full_name='api.Dex',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=1311,
  serialized_end=1953,
  methods=[
  _descriptor.MethodDescriptor(
    name='CreateClient',
    full_name='api.Dex.CreateClient',
    index=0,
    containing_service=None,
    input_type=_CREATECLIENTREQ,
    output_type=_CREATECLIENTRESP,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='UpdateClient',
    full_name='api.Dex.UpdateClient',
    index=1,
    containing_service=None,
    input_type=_UPDATECLIENTREQ,
    output_type=_UPDATECLIENTRESP,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='DeleteClient',
    full_name='api.Dex.DeleteClient',
    index=2,
    containing_service=None,
    input_type=_DELETECLIENTREQ,
    output_type=_DELETECLIENTRESP,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='CreatePassword',
    full_name='api.Dex.CreatePassword',
    index=3,
    containing_service=None,
    input_type=_CREATEPASSWORDREQ,
    output_type=_CREATEPASSWORDRESP,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='UpdatePassword',
    full_name='api.Dex.UpdatePassword',
    index=4,
    containing_service=None,
    input_type=_UPDATEPASSWORDREQ,
    output_type=_UPDATEPASSWORDRESP,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='DeletePassword',
    full_name='api.Dex.DeletePassword',
    index=5,
    containing_service=None,
    input_type=_DELETEPASSWORDREQ,
    output_type=_DELETEPASSWORDRESP,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ListPasswords',
    full_name='api.Dex.ListPasswords',
    index=6,
    containing_service=None,
    input_type=_LISTPASSWORDREQ,
    output_type=_LISTPASSWORDRESP,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetVersion',
    full_name='api.Dex.GetVersion',
    index=7,
    containing_service=None,
    input_type=_VERSIONREQ,
    output_type=_VERSIONRESP,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ListRefresh',
    full_name='api.Dex.ListRefresh',
    index=8,
    containing_service=None,
    input_type=_LISTREFRESHREQ,
    output_type=_LISTREFRESHRESP,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='RevokeRefresh',
    full_name='api.Dex.RevokeRefresh',
    index=9,
    containing_service=None,
    input_type=_REVOKEREFRESHREQ,
    output_type=_REVOKEREFRESHRESP,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_DEX)

DESCRIPTOR.services_by_name['Dex'] = _DEX

# @@protoc_insertion_point(module_scope)



import unittest
from dex_k8s_client.k8s.cluster import Dex_K8S_Cluster

class Dex_K8S_Cluster_Test(unittest.TestCase):
    """Tests for `k8s/cluster.py`."""

    def test_create_cluster_instance(self):
        """ Test creating a new Dex_K8S_Cluster instance """
        cluster = Dex_K8S_Cluster('name', 'ca_cert', 'api_url')
        self.assertIsInstance(cluster, Dex_K8S_Cluster)



from dex_k8s_client.dexidp.dex import DEX_VERSION

class _Dex_K8S_OAuth2_Settings(object):
    """
    Store parameters for making OAuth2 requests.
    """
    def __init__(self, redirect_uri, response_type, scope, access_type, prompt):

        self.redirect_uri  = redirect_uri
        self.response_type = response_type
        self.scope         = scope
        self.access_type   = access_type
        self.prompt        = prompt

class Dex_K8S_Connector(object):
    """
    Class object for storing connection attributes for an instance
    of Dex running on Kubernetes.
    """
    def __init__(self, host, grpc_port, https_port, client_cert, client_key,
        ca_cert, issuer_url, version=DEX_VERSION, oauth2_params={}):

        self.base_url    = 'https://{}:{}'.format(host, https_port)

        # Dex parameters
        self.host        = host
        self.grpc_port   = grpc_port
        self.https_port  = https_port
        self.client_cert = client_cert
        self.client_key  = client_key
        self.ca_cert     = ca_cert
        self.issuer_url  = issuer_url
        self.version     = version
        self.auth_url    = '{}/auth'.format(self.base_url)
        self.token_url   = '{}/token'.format(self.base_url)
        self.keys_url    = '{}/keys'.format(self.base_url)


        # Set the redirect URI
        redirect_uri = oauth2_params.get('redirect_uri',
            'https://{0}:{1}/callback'.format(self.host, self.https_port))

        # OAuth2 parameters
        self.oauth2 = _Dex_K8S_OAuth2_Settings(
            redirect_uri  = redirect_uri,
            response_type = oauth2_params.get('response_type', 'code'),
            scope         = oauth2_params.get('scope', 'openid groups profile email offline_access'),
            access_type   = oauth2_params.get('access_type', 'offline'),
            prompt        = oauth2_params.get('prompt', 'select_account')
        )



from dex_k8s_client.k8s.cluster import Dex_K8S_Cluster
from dex_k8s_client.k8s.connector import Dex_K8S_Connector

class Dex_K8S_Settings(object):
    """
    Class object for storing client settings.
    """
    def __init__(self, cluster, connector):

        # Enforce class instance requirements
        if not isinstance(cluster, Dex_K8S_Cluster):
            raise Exception('`cluster` argument must be an instance of Dex_K8S_Cluster')
        if not isinstance(connector, Dex_K8S_Connector):
            raise Exception('`cluster` argument must be an instance of Dex_K8S_Connector')

        self.cluster = cluster
        self.dex     = connector



class Dex_K8S_Bindings_Modules(object):
    """
    Class object for storing proto/grpc module paths.
    """
    def __init__(self, dex_version, mod_base):
        self.proto = '{}.v{}_pb2'.format(mod_base, dex_version.replace('.', '_'))
        self.grpc  = '{}.v{}_pb2_grpc'.format(mod_base, dex_version.replace('.', '_'))

class Dex_K8S_Bindings(object):
    """
    Class object for handling proto bindings for a particular
    version of the Dex API.
    """
    def __init__(self, settings, mod_base='dex_k8s_client.dexidp.dex.api'):
        self.version = settings.dex.version
        self.modules = Dex_K8S_Bindings_Modules(settings.dex.version, mod_base)



import unittest
from unittest import mock
from importlib import import_module
from parameterized import parameterized
from grpc_api_client.client import gRPC_API_Client
from grpc_api_client.grpc.api import gRPC_API_Path, gRPC_API_Response
from dex_k8s_client.client import Dex_K8S_Client
from dex_k8s_client.settings_test import make_settings

def make_fields(method):
    """
    Generate test fields for a Dex gRPC method.
    """
    settings = make_settings()
    test_fields = import_module('dex_k8s_client.dexidp.dex.api.v{0}_pb2_test_fields'.format(
        settings.dex.version.replace('.', '_')
    ))
    return getattr(test_fields, method.name)

def make_client(connect_grpc=True):
    """
    Convenience method for generating a test client class.
    """
    settings = make_settings()
    client_object = Dex_K8S_Client(settings.cluster, settings.dex)
    if connect_grpc:
        client_object.grpc_connect()
    return client_object

def generate_grpc_methods():
    """
    Generate an argument to pass to parameterized gRPC method tests
    """

    # Generate a mock client
    client = make_client()

    # Get all API methods and return a list of names
    method_tests = []
    for name, method in client.grpc.api:
        method_tests.append([name, method])
    return method_tests

class Mock_gRPC_API_Path(gRPC_API_Path):
    """
    Class for mocking a gRPC API path.
    """
    def __call__(self, **kwargs):
        import dex_k8s_client.dexidp.dex.api.v2_14_0_pb2 as dex_pb2

        response = {
            'CreateClient': dex_pb2.CreateClientResp(),
            'UpdateClient': dex_pb2.UpdateClientResp(),
            'DeleteClient': dex_pb2.CreateClientResp(),
            'CreatePassword': dex_pb2.CreatePasswordResp(),
            'UpdatePassword': dex_pb2.UpdatePasswordResp(),
            'DeletePassword': dex_pb2.DeletePasswordResp(),
            'GetVersion': dex_pb2.VersionResp(),
            'ListRefresh': dex_pb2.ListRefreshResp(),
            'RevokeRefresh': dex_pb2.RevokeRefreshResp(),
            'ListPasswords': dex_pb2.ListPasswordResp()
        }.get(self.name)

        return gRPC_API_Response(response, self.output.handler, self.input_fields())

@mock.patch('grpc_api_client.grpc.api.gRPC_API_Path', new=Mock_gRPC_API_Path)
class Dex_K8S_Client_Test(unittest.TestCase):
    """Tests for `client.py`."""

    def test_create_client(self):
        """ Test creating a new client interface. """
        client = make_client()
        self.assertIsInstance(client, Dex_K8S_Client)

    def test_client_connect(self):
        """ Test creating a new client interface and connecting. """
        client = make_client()
        client.grpc_connect()
        self.assertIsInstance(client.grpc, gRPC_API_Client)

    @parameterized.expand(generate_grpc_methods())
    def test_grpc_method(self, name, method):
        client = make_client()
        output_class = getattr(client.grpc.api, name).output.handler
        response = getattr(client.grpc.api, name)(**make_fields(method))
        self.assertIsInstance(response.protobuf, output_class)



from grpc_api_client.client import gRPC_API_Client

from dex_k8s_client.settings import Dex_K8S_Settings
from dex_k8s_client.bindings import Dex_K8S_Bindings
from dex_k8s_client.oauth2.client import Dex_K8S_OAuth2_Client
from dex_k8s_client.dexidp.dex import DEX_VERSION

class Dex_K8S_Client(object):
    """
    Client class for interacting with a Dex API server.
    """
    def __init__(self, cluster, connector):

        # Store connection settings
        self.settings = Dex_K8S_Settings(cluster, connector)

        # Create a new bindings instance
        self.bindings = Dex_K8S_Bindings(self.settings)

        # Create a new gRPC/OAuth2 client
        self.grpc     = gRPC_API_Client(
            self.bindings.modules.proto,
            self.bindings.modules.grpc
        )
        self.oauth2   = Dex_K8S_OAuth2_Client(self.settings)

    def grpc_connect(self):
        """ Open the connection to the Dex gRPC API server. """
        self.grpc.connect(self.settings.dex.host, self.settings.dex.grpc_port,
            ca_cert     = self.settings.dex.ca_cert,
            client_cert = self.settings.dex.client_cert,
            client_key  = self.settings.dex.client_key
        )

    def grpc_disconnect(self):
        """ Close connection to gRPC server """
        self.grpc.disconnect()

    def server_version(self):
        """
        Return the Dex server version for the client to inspect.
        """
        return self.grpc.api.GetVersion()

    def get_kubeconfig(self, client_id, client_secret, user_email, user_password):
        """
        Get the kubeconfig for a particular cluster for a user.
        """
        return self.oauth2.get_kubeconfig(
            client_id, client_secret, user_email, user_password)

    def get_token(self, client_id, client_secret, user_email, user_password):
        """
        Get a OAuth2 token for a user.
        """
        return self.oauth2.get_token(
            client_id, client_secret, user_email, user_password)

    def authorize_client(self, client_id):
        """
        Make an authorization request on behalf of a client.
        """
        return self.oauth2._authorize(client_id)

    def delete_client(self, client_id):
        """
        Delete an existing client.
        """
        return self.grpc.api.DeleteClient(id=client_id)

    def create_client(self, **kwargs):
        """
        Create a new client.
        """
        return self.grpc.api.CreateClient(client={
            'id': kwargs.get('id'),
            'secret': kwargs.get('secret'),
            'redirect_uris': kwargs.get('redirect_uris'),
            'trusted_peers': kwargs.get('trusted_peers'),
            'public': kwargs.get('public'),
            'name': kwargs.get('name'),
            'logo_url': kwargs.get('logo_url', None)
        })

    def update_client(self, **kwargs):
        """
        Update an existing client.
        """
        return self.grpc.api.UpdateClient(**kwargs)

    def create_password(self, **kwargs):
        """
        Create a new password.
        """
        return self.grpc.api.CreatePassword(password=kwargs)

    def delete_password(self, email):
        """
        Delete a password.
        """
        return self.grpc.api.DeletePassword(email=email)

    def update_password(self, **kwargs):
        """
        Update an existing password.
        """
        return self.grpc.api.UpdatePassword(**kwargs)

    def list_passwords(self):
        """
        Return a list of passwords.
        """
        return self.grpc.api.ListPasswords()

    def list_refresh(self, client_id, client_secret, user_email, user_password):
        """
        List refresh tokens for a user.
        """

        # Get the user's token first
        user_token = self.get_token(
            client_id, client_secret, user_email, user_password)

        return self.grpc.api.ListRefresh(user_id=user_token.decode()['sub'])

    def revoke_refresh(self, client_id, client_secret, user_email, user_password):
        """
        Revoke a refresh token for a user.
        """

        # Get the user's token first
        user_token = self.get_token(
            client_id, client_secret, user_email, user_password)

        return self.grpc.api.RevokeRefresh(user_id=user_token.decode()['sub'], client_id=client_id)



from yaml import load as load_yaml
from io import StringIO

TEMPLATE = """
apiVersion: v1
clusters:
- cluster:
    certificate-authority-data: {cluster_ca}
    server: "{api_url}"
  name: {cluster_name}
contexts:
- context:
    cluster: {cluster_name}
    user: {email}
  name: {cluster_name}
current-context: {cluster_name}
kind: Config
preferences: {{}}
users:
- name: {email}
  user:
    auth-provider:
      config:
        client-id: {client_id}
        client-secret: {client_secret}
        id-token: {id_token}
        idp-issuer-url: {issuer_url}
        refresh-token: {refresh_token}
      name: oidc
"""

class Dex_K8S_KubeConfig(object):
    """
    Class object for constructing a kubeconfig.
    """
    @staticmethod
    def from_template(**kwargs):
        """
        Construct and return a kubeconfig from a template.
        """
        return load_yaml(StringIO(TEMPLATE.format(**kwargs)))



import unittest
from dex_k8s_client.k8s.connector import Dex_K8S_Connector

class Dex_K8S_Connector_Test(unittest.TestCase):
    """Tests for `k8s/connector.py`."""

    def test_create_connector_instance(self):
        """ Test creating a new Dex_K8S_Connector instance """
        connector = Dex_K8S_Connector('host', 'grpc_port', 'https_port',
            'client_cert', 'client_key', 'ca_cert', 'issuer_url',
            'version', oauth2_params={})

        self.assertIsInstance(connector, Dex_K8S_Connector)



import unittest
from dex_k8s_client.bindings import Dex_K8S_Bindings
from dex_k8s_client.settings_test import make_settings

class Dex_K8S_Bindings_Test(unittest.TestCase):
    """Tests for `bindings.py`."""

    def test_create_bindings(self):
        """ Test creating a new gRPC API bindings object. """
        bindings = Dex_K8S_Bindings(make_settings())
        self.assertIsInstance(bindings, Dex_K8S_Bindings)



class Dex_K8S_Cluster(object):
    """
    Class object for passing to the client class for defining
    and storing cluster attributes.
    """
    def __init__(self, name, ca_cert, api_url):
        self.name    = name
        self.ca_cert = ca_cert
        self.api_url = api_url



import unittest
from dex_k8s_client.oauth2.parser import Dex_K8S_OAuth2_Parser as parser
from dex_k8s_client.settings_test import make_settings
from dex_k8s_client.oauth2.client_test import example_auth_response_html, \
    example_login_url

class Dex_K8S_OAuth2_Parser_Test(unittest.TestCase):
    """Tests for `oauth2/parser.py`."""

    def test_parse_ldap_auth_uri(self):
        """ Test parsing LDAP URI from auth page """
        auth_uri = parser.get_ldap_authentication_uri(example_auth_response_html, make_settings())
        self.assertEqual(auth_uri, example_login_url)



#!/usr/bin/python
from setuptools import setup, find_packages

# Import the module version
from dex_k8s_client import __version__

with open('requirements-test.txt', 'r') as f:
    tests_require = [x.rstrip() for x in f.readlines()]

with open('requirements.txt', 'r') as f:
    install_requires = [x.rstrip() for x in f.readlines()]

# Run the setup
setup(
    name             = 'dex_k8s_client',
    version          = __version__,
    description      = 'Python bindings for interacting with a Dex server running on Kubernetes.',
    long_description = open('DESCRIPTION.rst').read(),
    author           = 'David Taylor',
    author_email     = 'djtaylor13@gmail.com',
    url              = 'http://github.com/djtaylor/python-dex-k8s-client',
    license          = 'GPLv3',
    install_requires = install_requires,
    test_suite       = 'nose.collector',
    tests_require    = tests_require,
    packages         = find_packages(),
    keywords         = 'grpc rpc api dex k8s kubernetes',
    classifiers      = [
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Natural Language :: English',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Topic :: Software Development :: User Interfaces',
        'Topic :: Terminals',
    ]
)



from passlib.hash import bcrypt

"""
Define test fields for making requests to the Dex server
during test runs. These fields should map to what is defined
in the proto definition for this API version.
"""

CreateClient = {
    'client': {
        'id': 'testclient',
        'secret': 'testsecret',
        'redirect_uris': ['http://one.com', 'http://two.com'],
        'trusted_peers': ['localhost', '127.0.0.1'],
        'public': True,
        'name': 'Test Client',
        'logo_url': ''
    }
}

DeleteClient = {
    'id': 'testclient'
}

UpdateClient = {
    'id': 'testclient',
    'redirect_uris': ['http://three.com', 'http://four.com'],
    'trusted_peers': ['updatedpeer1', 'updatedpeer2'],
    'name': 'Test Client 2',
}


CreatePassword = {
    'password': {
        'email': 'user@domain.com',
        'hash': bcrypt.hash("password").encode(),
        'username': 'Test Client 2',
        'user_id': 'testclient'
    }
}

UpdatePassword = {
    'email': 'user@domain.com',
    'new_hash': bcrypt.hash("password").encode(),
    'new_username': 'Test Client 2'
}

DeletePassword = {
    'email': 'user@domain.com'
}

ListPasswords = {}
GetVersion = {}

ListRefresh = {
    'user_id': 'testclient'
}

RevokeRefresh = {
    'user_id': 'testclient',
    'client_id': 'testclient'
}



import unittest
from importlib import import_module
from dex_k8s_client.settings import Dex_K8S_Settings
from dex_k8s_client.k8s.cluster import Dex_K8S_Cluster
from dex_k8s_client.k8s.connector import Dex_K8S_Connector

example_client_cert = 'docker_files/dex_tls/client.crt'
example_client_key = 'docker_files/dex_tls/client.key'
example_ca_cert = 'docker_files/dex_tls/ca.crt'

def make_settings():
    """
    Convenience method for generating a test client class.
    """

    cluster = Dex_K8S_Cluster('name', example_ca_cert, 'api_url')
    connector = Dex_K8S_Connector('localhost', '5557', '5556',
        example_client_cert, example_client_key, example_ca_cert, 'issuer_url',
        '2.14.0', oauth2_params={})

    return Dex_K8S_Settings(cluster, connector)

class Dex_K8S_Settings_Test(unittest.TestCase):
    """Tests for `settings.py`."""

    def test_create_client_settings(self):
        """ Test creating a new client settings object. """
        settings = make_settings()
        self.assertIsInstance(settings, Dex_K8S_Settings)



import jwt
import json

class Dex_K8S_OAuth2_Token(object):
    """
    Class for managing tokens retrieved from Dex.
    """
    def __init__(self, token, client_id, session, settings):
        self._token     = token
        self._client_id = client_id
        self._session   = session
        self._settings  = settings
        self._verify    = getattr(settings, 'ca_cert', True)

    def _get_jwt_pubkey(self):
        """
        Get the JWT public key from the server.
        """
        pubkey_response = self._session.get(self._settings.dex.keys_url, verify=self._verify)
        pubkey_json = pubkey_response.json()['keys'][0]

        return jwt.algorithms.RSAAlgorithm.from_jwk(json.dumps(pubkey_json))

    def decode(self):
        """
        Decode the `id_token` key of an OAuth2 token.
        """
        pubkey = self._get_jwt_pubkey()

        # Decode it
        return jwt.decode(self._token['id_token'], pubkey, audience=self._client_id, algorithm='RS256')

    def json(self):
        """
        Return the JSON object for the full token.
        """
        return self._token



import unittest
from unittest import mock
from dex_k8s_client.settings_test import make_settings
from dex_k8s_client.oauth2.client import Dex_K8S_OAuth2_Client
from dex_k8s_client.client_test import make_client

example_token = {
    'access_token': 'bkhafadfvHJDAF',
    'token_type': 'bearer',
    'expires_in': 57248,
    'refresh_token': 'fgadw48ay5nkctnzksgnkzsd',
    'id_token': 'kycrkawgnctjsgtbkszc',
    'expires_at': 1551648182.857075
}

example_state = 'QTxEd0QoMKgIieANLl67i7iaSkjxw0'
example_auth_url = ''.join([
    'https://localhost:5556/auth?response_type=code&client_id=client',
    '&redirect_uri=https%3A%2F%2Flocalhost%3A5556%2Fcallback',
    '&scope=openid+groups+profile+email+offline_access',
    '&state={}&access_type=offline'.format(example_state),
    '&prompt=select_account',
])

example_callback_url = ''.join([
    '<a href="https://kubernetes:5556/callback?code=irgenlidu73igudeuobi3pd4d&',
    'amp;state=TpKCxqYwepp9tFzHjmqb40caATmE9C">See Other</a>.'
])

example_approval_location = '/approval?req=jgpebqvb6qv4tidmezzayckrt'
example_approval_url = 'https://localhost:5556{}'.format(example_approval_location)
example_login_url = 'https://localhost:5556/auth/ldap?req=jgpebqvb6qv4tidmezzayckrt'
example_auth_response_html = """
<!DOCTYPE html>
<html>
  <head>
    <meta charset="utf-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge,chrome=1">
    <title>dex</title>
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <link href="https://localhost:5556/static/main.css" rel="stylesheet">
    <link href="https://localhost:5556/theme/styles.css" rel="stylesheet">
    <link rel="icon" href="https://localhost:5556/theme/favicon.png">
  </head>

  <body class="theme-body">
    <div class="theme-navbar">
      <div class="theme-navbar__logo-wrap">
        <img class="theme-navbar__logo" src="https://localhost:5556/theme/logo.png">
      </div>
    </div>

    <div class="dex-container">



<div class="theme-panel">
  <h2 class="theme-heading">Log in to dex </h2>
  <div>

      <div class="theme-form-row">
        <a href="/auth/local?req=jgpebqvb6qv4tidmezzayckrt" target="_self">
          <button class="dex-btn theme-btn-provider">
            <span class="dex-btn-icon dex-btn-icon--local"></span>
            <span class="dex-btn-text">Log in with Email</span>
          </button>
        </a>
      </div>

      <div class="theme-form-row">
        <a href="/auth/ldap?req=jgpebqvb6qv4tidmezzayckrt" target="_self">
          <button class="dex-btn theme-btn-provider">
            <span class="dex-btn-icon dex-btn-icon--ldap"></span>
            <span class="dex-btn-text">Log in with OpenLDAP</span>
          </button>
        </a>
      </div>

  </div>
</div>

    </div>
  </body>
</html>
"""

class Mock_OAuth2Session(object):
    """
    Mock OAuth2 session for testing.
    """
    class HTTP_Response(object):
        def __init__(self, status_code, text, headers={}):
            self.status_code = status_code
            self.text = text
            self.headers = headers

    def __init__(self, *args, **kwargs):
        pass

    def close(self, *args, **kwargs):
        return True

    def fetch_token(self, *args, **kwargs):
        return example_token

    def authorization_url(self, *args, **kwargs):
        return example_auth_url, example_state

    def post(self, *args, **kwargs):

        if args[0] == example_login_url:
            return Mock_OAuth2Session.HTTP_Response(303, example_callback_url,
                headers={
                    'Location': example_approval_location,
                    'Date': 'Sat, 02 Mar 2019 21:46:07 GMT',
                    'Content-Length': '0'
                })

    def get(self, *args, **kwargs):

        if args[0] == example_auth_url:
            return Mock_OAuth2Session.HTTP_Response(200, example_auth_response_html)

        if args[0] == example_approval_url:
            return Mock_OAuth2Session.HTTP_Response(200, example_callback_url)


class Dex_K8S_OAuth2_Client_Test(unittest.TestCase):
    """Tests for `oauth2/client.py`."""

    def test_class_instance(self):
        """ Test creating a new instance of Dex_K8S_OAuth2_Client. """
        oauth2_client = Dex_K8S_OAuth2_Client(make_settings())
        self.assertIsInstance(oauth2_client, Dex_K8S_OAuth2_Client)

    @mock.patch('dex_k8s_client.oauth2.client.OAuth2Session', new=Mock_OAuth2Session)
    def test_get_token(self, *args, **kwargs):
        """ Test getting an OAuth2 token. """
        client = make_client()
        token = client.get_token('test', 'test', 'test@test.com', 'test')
        self.assertEqual(token, example_token)

    @mock.patch('dex_k8s_client.oauth2.client.OAuth2Session', new=Mock_OAuth2Session)
    def test_get_kubeconfig(self, *args, **kwargs):
        """ Test getting an OAuth2 token. """
        client = make_client()
        kubeconfig = client.get_kubeconfig('test', 'test', 'test@test.com', 'test')
